/***************************************************************************
 *   Copyright (C) 2006 by Julien Lemoine, Simon Viennot                   *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "widgetoptions.h"

//pour les variables statiques des classes calcul et WidgetHorloge
#include "calcul.h"
#include "widgethorloge.h"
#include "tirage.h"
#include "widgetprincipal.h"

#include <QtGui>
#include <fstream>

WidgetOptions::WidgetOptions(QWidget* parent) : QDialog( parent) {
	//titre de la fentre
	setWindowTitle(QString("Options"));
	
	//boutons  cocher pour les diverses canonisations
	QLabel *titrecano1 = new QLabel(this);
	titrecano1->setText("Activer \"2+2=2x2\" :");
	titrecano1->setFont( font_monospace );
	cano1 = new QCheckBox(this);
	
	QLabel *titrecano2 = new QLabel(this);
	titrecano2->setText("Activer \"sous-calcul\" :");
	titrecano2->setFont( font_monospace );
	cano2 = new QCheckBox(this);
	
	QLabel *titrecano3 = new QLabel(this);
	titrecano3->setText("Activer \"factorisation\" :");
	titrecano3->setFont( font_monospace );
	cano3 = new QCheckBox(this);
	
	//rglage du temps de rflexion
	QLabel *titreTemps = new QLabel(this);
	titreTemps->setText("Temps de rflexion :");
	boxTemps = new QSpinBox(this);
	boxTemps->setRange(min_temps_reflexion,max_temps_reflexion);
	boxTemps->setFixedSize(boxTemps->sizeHint());
	
	//rglage du nombre de plaques
	QLabel *titreNbPlaque = new QLabel(this);
	titreNbPlaque->setText("Nombre de plaques :");
	boxNbPlaque = new QSpinBox(this);
	boxNbPlaque->setRange(min_nb_plaques,max_nb_plaques);
	boxNbPlaque->setFixedSize(boxNbPlaque->sizeHint());
	
	//rglage de la valeur minimale du compte
	QLabel *titreCompte = new QLabel(this);
	titreCompte->setText("Valeur minimale du compte :");
	boxCompte = new QSpinBox(this);
	boxCompte->setRange(min_min_compte,max_min_compte);
	boxCompte->setFixedSize(boxCompte->sizeHint());
	
	//rglage de la valeur maximale du compte
	QLabel *titreCompte2 = new QLabel(this);
	titreCompte2->setText("Valeur maximale du compte :");
	boxCompte2 = new QSpinBox(this);
	boxCompte2->setRange(min_max_compte,max_max_compte);
	boxCompte2->setFixedSize(boxCompte2->sizeHint());
	
	//bouton pour rtablir les options par dfaut
	pbDefaut = new QPushButton( "Options par dfaut", this );
	pbDefaut->setFont( font_normal );
	connect( pbDefaut, SIGNAL( clicked() ), this, SLOT( modifDefaut() ) );
	
	//bouton "OK" pour fermer la fentre
	pbOk = new QPushButton( "OK", this );
	pbOk->setFont( font_normal );
	//Le bouton "OK" a le focus au lancement de la fentre
	pbOk->setFocus();
	connect( pbOk, SIGNAL( clicked() ), this, SLOT( modifOk() ) );
	
	//potisionnement des widgets
	layout = new QGridLayout;
	setMaximumHeight( 150 );
	setMaximumWidth( 300 );
	layout->addWidget(titrecano1, 0, 0);
	layout->addWidget(cano1, 0, 1);
	layout->addWidget(titrecano2, 1, 0);
	layout->addWidget(cano2, 1, 1);
	layout->addWidget(titrecano3, 2, 0);
	layout->addWidget(cano3, 2, 1);
	layout->addWidget(titreTemps, 3, 0);
	layout->addWidget(boxTemps, 3, 1);
	layout->addWidget(titreNbPlaque, 4, 0);
	layout->addWidget(boxNbPlaque, 4, 1);
	layout->addWidget(titreCompte, 5, 0);
	layout->addWidget(boxCompte, 5, 1);
	layout->addWidget(titreCompte2, 6, 0);
	layout->addWidget(boxCompte2, 6, 1);
	layout->addWidget(pbDefaut, 7, 0);
	layout->addWidget(pbOk, 7, 1);
	
	setLayout(layout);
	
	//initialisation des etats
	if (calcul::cano1_actif==true) {
		cano1->setCheckState(Qt::Checked);
	} else {
		cano1->setCheckState(Qt::Unchecked);
	}
	
	if (calcul::cano2_actif==true) {
		cano2->setCheckState(Qt::Checked);
	} else {
		cano2->setCheckState(Qt::Unchecked);
	}
	
	if (calcul::cano3_actif==true) {
		cano3->setCheckState(Qt::Checked);
	} else {
		cano3->setCheckState(Qt::Unchecked);
	}
	
	boxTemps->setValue(WidgetHorloge::nb_sec);
	boxNbPlaque->setValue(tirage::nb_plaques);
	boxCompte->setValue(tirage::compte_minimal);
	boxCompte2->setValue(tirage::compte_maximal);
}

//les modifs sont valides
void WidgetOptions::modifOk() {
	//modification des paramtres
	calcul::cano1_actif=cano1->isChecked();
	calcul::cano2_actif=cano2->isChecked();
	calcul::cano3_actif=cano3->isChecked();
	WidgetHorloge::nb_sec=boxTemps->value();
	tirage::nb_plaques=boxNbPlaque->value();
	if(boxCompte->value()<boxCompte2->value()){		//cas normal o la valeur rentre pour le compte minimal est plus petite que l'autre
		tirage::compte_minimal=boxCompte->value();
		tirage::compte_maximal=boxCompte2->value();
	}
	else{							//cas o l'utilisateur est un boulet
		tirage::compte_minimal=boxCompte2->value();
		tirage::compte_maximal=boxCompte->value();
	}
	//criture du fichier de sauvegarde des options
	QString le_chemin=QDir::homePath();
	QString le_fichier=QString("/.kitsune.options");
	le_chemin.append(le_fichier);
	string filename=le_chemin.toStdString();
	ofstream f_options(filename.c_str());
	if (f_options.is_open()){
		f_options << calcul::cano1_actif << endl;
		f_options << calcul::cano2_actif << endl;
		f_options << calcul::cano3_actif << endl;
		f_options << WidgetHorloge::nb_sec << endl;
		f_options << tirage::nb_plaques << endl;
		f_options << tirage::compte_minimal << endl;
		f_options << tirage::compte_maximal << endl;
		f_options.close();
	}
	//fermeture du widget des options
	close();
}

//Pour remettre les options  leur valeur par dfaut
void WidgetOptions::modifDefaut() {
	cano1->setCheckState(Qt::Checked);
	cano2->setCheckState(Qt::Checked);
	cano3->setCheckState(Qt::Checked);
	boxTemps->setValue(50);
	boxNbPlaque->setValue(6);
	boxCompte->setValue(100);
	boxCompte2->setValue(999);
}
