/***************************************************************************
 *   Copyright (C) 2006 by Julien Lemoine, Simon Viennot                   *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "widgetplaque.h"
#include "tirage.h"

#include <QtGui>

//Cration d'un widget regroupant 1 Lcd et 2 pushbuttons pour les plaques
WidgetPlaque::WidgetPlaque( QWidget *parent, int numW, bool est_compteW)
        : QWidget( parent) {
	
	//init du num de widget et de variable tat est_compte
	num=numW;
	est_compte=est_compteW;
	
	//init des elements graphiques
	vboxLayout = new QVBoxLayout(this);
	vboxLayout->setSpacing(0);
	vboxLayout->setMargin(0);
	
	//un afficheur lcd
	lcd = new QLCDNumber(this);
	
	if (est_compte) {
		lcd->setNumDigits( 1 );
		lcd->setFixedSize( 32, 51 );
	} else {
		lcd->setNumDigits( 3 );
		lcd->setFixedHeight( 28 );
	}
	
	lcd->setSegmentStyle( QLCDNumber::Flat );
	lcd->setAutoFillBackground ( true );
	
	//couleurs, polices qui serviront en plusieurs endroits par la suite
	palette_orange = new QPalette();
	palette_orange->setColor( QPalette::Window, QColor( 255, 196, 128 ) );
	palette_blanc = new QPalette();
	palette_blanc->setColor( QPalette::Window, QColor( 255, 255, 255 ) );
	//deux pushbuttons dans un layout horizontal
	QString chMoins;
	QString chPlus;
	
	if (est_compte) {
		lcd->setPalette( *palette_orange );
		chMoins=QString(":/images/moins.png");
		chPlus=QString(":/images/plus.png");
	} else {
		lcd->setPalette( *palette_blanc );
		chMoins=QString(":/images/moins2.png");
		chPlus=QString(":/images/plus2.png");
	}
	
	QPixmap *pixMoins = new QPixmap( chMoins );
	QPixmap *pixPlus = new QPixmap( chPlus );
	
	hbox = new QWidget(this);
	hboxLayout = new QHBoxLayout(hbox);
	hboxLayout->setSpacing(0);
	hboxLayout->setMargin(0);
	
	pbMoins = new QPushButton( "", hbox);
	pbMoins->setFlat( true );
	pbMoins->setIcon( *pixMoins );
	pbMoins->setFixedSize( 15, 15 );
	pbMoins->setAutoRepeat ( true );
	
	pbPlus = new QPushButton( "", hbox);
	pbPlus->setFlat( true );
	pbPlus->setIcon( *pixPlus );
	pbPlus->setFixedSize( 15, 15 );
	pbPlus->setAutoRepeat ( true );
	
	hboxLayout->addWidget( pbMoins);
	hboxLayout->addWidget( pbPlus);
	
	vboxLayout->addWidget(lcd);
	vboxLayout->addWidget(hbox);
	
	//connections internes
	connect( pbMoins, SIGNAL(clicked()), this, SLOT(pbMoinsClicked()) );
	connect( pbPlus, SIGNAL(clicked()), this, SLOT(pbPlusClicked()) );
}

void WidgetPlaque::pbMoinsClicked() {
	emit change(num, FALSE);
}

void WidgetPlaque::pbPlusClicked() { 
	emit change(num, TRUE);
}

void WidgetPlaque::displayRefresh(int numW, int valeur) {
	//rception du signal si le numro de widget correspond
	//mthode un peu lourde mais qui permet de supprimer le nom des des widgets
	if(num==numW) {
		lcd->display(valeur);
		//on n'affiche que les plaques que l'on utilise
		if(numW>=tirage::nb_plaques && numW<10){
			hide();
		}
		else{
			show();
		}

	}
}
