/***************************************************************************
 *   Copyright (C) 2006 by Julien Lemoine, Simon Viennot                   *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "widgetpopup.h"

#include <QtGui>

//La fentre qui apparat quand on clique sur le menu "Licence" est diffrente des 4 autres fentres de ce type
//(elle possde deux boutons en plus), d'o le boolen "est_fenetre_licence" pour dterminer dans quel cas on se trouve
WidgetPopup::WidgetPopup( QWidget* parent, bool est_fenetre_licence, QString nomFichier, QString titreFenetre)
	 : QDialog( parent) {
	//titre de fentre
	if(titreFenetre!=QString("")) {
		setWindowTitle(titreFenetre);
	}
		
	//un layout vertical pour le QDialog WidgetPopup
	vboxPopup = new QVBoxLayout( this );
	vboxPopup->setSpacing(0);
	vboxPopup->setMargin(0);
	
	//deux layouts horizontaux dans lesquels on placera le texte pour l'un, le ou les boutons pour l'autre
	horizontalLayout1 = new QWidget( this );
	horizontalLayout2 = new QWidget( this );
	hboxLayout1 = new QHBoxLayout(horizontalLayout1);
	hboxLayout2 = new QHBoxLayout(horizontalLayout2);
	
	hboxLayout1->setSpacing(0);
	hboxLayout1->setMargin(0);
	hboxLayout2->setSpacing(0);
	hboxLayout2->setMargin(0);
	
	vboxPopup->addWidget( horizontalLayout1 );
	vboxPopup->addWidget( horizontalLayout2 );
	
	//le cadre de texte
	textEditPopup = new QTextEdit( horizontalLayout1 );
	
	//la taille du texte, dont dpend la taille de la fentre au lancement
	textEditPopup->setMinimumWidth(548);
	textEditPopup->setMinimumHeight(313);
	
	//paramtres du texte
	textEditPopup->setReadOnly( TRUE );
	textEditPopup->setFont( font_monospace );
	
	if(nomFichier!=QString("")) {
		QFile fichier(nomFichier);
		fichier.open( QIODevice::ReadOnly );
		QTextStream textestream( &fichier );
		textestream.setCodec("ISO 8859-1");			//textes en franais
		textEditPopup->setPlainText(textestream.readAll());	//lecture du texte du fichier
		fichier.close();
	}
	
	hboxLayout1->addWidget(textEditPopup);
	
	//les boutons "Licence", "Traduction", que l'on affiche ou non suivant le cas
	if (est_fenetre_licence){
		pbLicence = new QPushButton( "Licence", horizontalLayout2 );
		pbLicence->setFont( font_normal );
		pbTraduction = new QPushButton( "Traduction", horizontalLayout2 );
		pbTraduction->setFont( font_normal );
		hboxLayout2->addStretch(1);
		hboxLayout2->addWidget(pbLicence);
		hboxLayout2->addStretch(1);
		hboxLayout2->addWidget(pbTraduction);
	}
	
	//le bouton "OK" que l'on affiche de toute faon
	pbOk = new QPushButton( "OK", horizontalLayout2 );
	pbOk->setFont( font_normal );
	//Il capte le focus au lancement, ce qui permet de juste avoir  appuyer sur "Entre" pour sortir de la fentre
	pbOk->setFocus();
	hboxLayout2->addStretch(1);
	hboxLayout2->addWidget(pbOk);
	hboxLayout2->addStretch(1);
	
	//on ferme la fentre quand on clique sur "OK"
	connect( pbOk, SIGNAL( clicked() ), this, SLOT( close() ) );
	//fonctions reliant les boutons "Licence", "Traduction" et l'ouverture d'une nouvelle fentre.
	if(est_fenetre_licence){
		connect( pbLicence, SIGNAL( clicked() ), this, SLOT( ouvre_licence() ) );
		connect( pbTraduction, SIGNAL( clicked() ), this, SLOT( ouvre_traduction() ) );
	}
}

//fonction d'affichage d'une fentre contenant la GPL en anglais
//traduction ou non suivant boolen
void WidgetPopup::ouvre_licence() {
	lire_texte(false);
}

//fonction d'affichage d'une fentre contenant la GPL en franais
void WidgetPopup::ouvre_traduction() {
	lire_texte(true);
}

void WidgetPopup::lire_texte(bool traduction) {
	if(traduction) {
		WidgetPopup popup( this, false, QString(":/txt/gpl-fr.txt"), QString("Traduction de la Licence GNU GPL"));
		popup.exec();
	} else {
		WidgetPopup popup( this, false, QString(":/txt/gpl.txt"), QString("Licence originale (anglais) GNU GPL"));
		popup.exec();
	}
}
