/***************************************************************************
 *   Copyright (C) 2006 by Julien Lemoine, Simon Viennot                   *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "widgetprincipal.h"

#include <QtGui>

//---------------------Widget Principal----------------------------------------

Principal::Principal( QWidget *parent ) : QMainWindow( parent ) {
	setWindowTitle(QString("Kitsune"));
	
	//Barre de menus
	MenuBar = new QMenuBar( this );
	setMenuBar(MenuBar);
	MenuBar->setFont( font_normal );
	
	fichierMenu = new QMenu ( this );
	infoMenu = new QMenu ( this );
	
	optionsAction = new QAction( this );
	fileExitAction = new QAction( this );
	
	fichierMenu = menuBar()->addMenu("&Fichier");
	fichierMenu->addAction(optionsAction);
	fichierMenu->addAction(fileExitAction);
	fichierMenu->setFont( font_normal );
	
	helpAideAction = new QAction( this );
	helpLicenceAction = new QAction( this );
	helpAboutAction = new QAction( this );
	infoMenu = menuBar()->addMenu("&Info");
	infoMenu->setFont( font_normal );
	infoMenu->addAction(helpAideAction);
	infoMenu->addAction(helpLicenceAction);
	infoMenu->addAction(helpAboutAction);
	
	//initialisation des menus
	fileExitAction->setText( "&Quitter" );
	optionsAction->setText( "&Options" );
	helpAideAction->setText( "Ai&de" );
	helpLicenceAction->setText( "&Licence" );
	helpAboutAction->setText( "A &propos" ); 
	
	//fonctions de connexion signal<--->slot
	//fonctions pour la barre de menus
	connect( fileExitAction, SIGNAL( triggered() ), this, SLOT( close() ) );
	connect( optionsAction, SIGNAL( triggered() ), this, SLOT( options() ) );
	connect( helpAideAction, SIGNAL( triggered() ), this, SLOT( helpAide() ) );
	connect( helpLicenceAction, SIGNAL( triggered() ), this, SLOT( helpLicence() ) );
	connect( helpAboutAction, SIGNAL( triggered() ), this, SLOT( helpAbout() ) );
	
	//Pour la barre de statut, temps de calcul
	heure = new QTime();
	
	//Widget parent
	hbox = new QWidget( this );
	hboxLayout = new QHBoxLayout( hbox );
	
	hboxLayout->setSpacing(1);
	hboxLayout->setMargin(1);
	
	setCentralWidget(hbox);
	
	//Bote verticale contenant tous les lments de la gauche de la fentre
	vboxA = new QWidget( hbox );
	vboxLayoutA = new QVBoxLayout( vboxA );
	
	vboxA->setFixedWidth(288);
	vboxLayoutA->setSpacing(0);
	vboxLayoutA->setMargin(0);
	
	//cadre de texte pour les rsultats, colonne de droite
	textEditB = new QTextEdit( hbox );
	textEditB->setReadOnly( TRUE );
	textEditB->setFixedWidth(260);
	textEditB->setFont( font_monospace );
	
	//insertion des widgets dans le layout
	hboxLayout->addWidget( vboxA );
	hboxLayout->addWidget( textEditB );
	
	//Widgets aligns verticalement  gauche
	horLayout1 = new QWidget( vboxA );
	hboxLayout1 = new QHBoxLayout(horLayout1);
	hboxLayout1->setSpacing(0);
	hboxLayout1->setMargin(0);
	horLayout2 = new QWidget( vboxA );
	hboxLayout2 = new QHBoxLayout(horLayout2);
	hboxLayout2->setSpacing(0);
	hboxLayout2->setMargin(0);
	horLayout3 = new QWidget( vboxA );
	hboxLayout3 = new QHBoxLayout(horLayout3);
	hboxLayout3->setSpacing(0);
	hboxLayout3->setMargin(0);
	horLayout4 = new QWidget( vboxA );
	hboxLayout4 = new QHBoxLayout(horLayout4);
	hboxLayout4->setSpacing(0);
	hboxLayout4->setMargin(0);
	texte_renard = new QLabel( vboxA );
	horLayout5 = new QWidget( vboxA );
	hboxLayout5 = new QHBoxLayout(horLayout5);
	hboxLayout5->setSpacing(0);
	hboxLayout5->setMargin(0);
	BarreStatut = new QStatusBar ( vboxA );
	
	//Espaces qui servent plus loin pour placer ou centrer les lments
	spacerItemH = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
	spacerItemV = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);
	
	//insertion des widgets dans le layout
	vboxLayoutA->addItem(spacerItemV);
	vboxLayoutA->addWidget( horLayout1 );
	vboxLayoutA->addWidget( horLayout2 );
	vboxLayoutA->addWidget( horLayout3 );
	vboxLayoutA->addWidget( horLayout4 );
	vboxLayoutA->addWidget( texte_renard );
	vboxLayoutA->addWidget( horLayout5 );
	vboxLayoutA->addWidget( BarreStatut );
	vboxLayoutA->addItem(spacerItemV);
	
	//premire ligne : le compte + les modifieurs
	//WidgetPlaque 10, 11 et 12
	hboxLayout1->addItem( spacerItemH );
	
	for(int Wi=0; Wi<3; Wi++) {
		WidgetPlaque *plaque= new WidgetPlaque(horLayout1, Wi+10, true);
		hboxLayout1->addWidget( plaque );
		connect( this, SIGNAL(refresh(int, int)), plaque, SLOT(displayRefresh(int, int)) );
		connect( plaque, SIGNAL(change(int, bool)), this, SLOT(compte_change(int, bool)) );
	}
	
	hboxLayout1->addItem( spacerItemH );
	
	//deuxime ligne : les plaques + les modifieurs
	horLayout2->setFixedHeight( 43 );
	
	//on "oublie" les widgets crs (le moteur Qt ne les oublie pas) et on ne communique que par signaux
	//conformment au tutorial Qt4 "Qt Tutorial 7"
	for(int Wi=0; Wi<6; Wi++) {
		WidgetPlaque *plaque= new WidgetPlaque(horLayout2, Wi, false);
		hboxLayout2->addWidget( plaque );
		connect( this, SIGNAL(refresh(int, int)), plaque, SLOT(displayRefresh(int, int)) );
		connect( plaque, SIGNAL(change(int, bool)), this, SLOT(plaque_change(int, bool)) );
	}
	
	//troisime ligne : les boutons
	horLayout3->setFixedHeight( 27 );
	
	pbRes = new QPushButton( "Rsoudre", horLayout3 );
	pbAlea = new QPushButton( "Alatoire", horLayout3 );
	pbZero = new QPushButton( "Mise  zro", horLayout3 );
	
	pbRes->setFont( font_normal );
	pbAlea->setFont( font_normal );
	pbZero->setFont( font_normal );
	
	//au dmarrage de l'application, le focus est positionn sur le bouton "alatoire"
	pbAlea->setFocus();
	
	hboxLayout3->addWidget( pbRes );
	hboxLayout3->addWidget( pbAlea );
	hboxLayout3->addWidget( pbZero );
	
	//quatrime ligne : image du renard
	horLayout4->setFixedHeight(150);
	
	pixmapLabelRenard = new QLabel( horLayout4 );
	renard_normal = new QPixmap(":/images/normal-150x150.png");
	renard_heureux = new QPixmap(":/images/heureux-150x150.png");
	renard_triste = new QPixmap(":/images/triste-150x150.png");
	pixmapLabelRenard->setPixmap(*renard_normal);
	pixmapLabelRenard->setFixedSize( 150, 150 );
	
	hboxLayout4->addItem(spacerItemH);
	hboxLayout4->addWidget(pixmapLabelRenard);
	hboxLayout4->addItem(spacerItemH);
	
	//cinquime ligne : le texte
	texte_renard->setFont( font_gros );
	texte_renard->setAlignment( Qt::AlignHCenter );
	texte_renard->setText( texte_renard_accueil );
	
	//sixime ligne : l'horloge
	horLayout5->setFixedHeight( 15 );
	
	horloge=new WidgetHorloge( horLayout5 );
	connect( horloge, SIGNAL(TempsEpuise()), this, SLOT(affiche_solutions()) );
	
	hboxLayout5->addWidget(horloge);
	
	//septime ligne : la barre de statut
	BarreStatut->setSizeGripEnabled ( false );
	BarreStatut->setFont( font_normal );
	
	//pour avoir du vrai alatoire, pas toujours les mmes valeurs
	srand (time (NULL));
	
	//fonctions pour les boutons
	connect( pbRes, SIGNAL(clicked()), this, SLOT(resoudre()) );
	connect( pbAlea, SIGNAL(clicked()), this, SLOT(aleatoire()) );
	connect( pbZero, SIGNAL(clicked()), this, SLOT(mise_a_zero()) );

	//init et connection de la classe jeu
	J =new jeu;
	calc_en_cours=false; alea=false;
	connect( J, SIGNAL(finished()), this, SLOT(fin_calcul()) );

	//initialisation des paramtres
	//"demarrage" sert  ne pas modifier le texte d'accueil,  l'ouverture de l'application
	demarrage=true;
	//chargement du fichier d'options
	QString le_chemin=QDir::homePath();
	QString le_fichier=QString("/.kitsune.options");
	le_chemin.append(le_fichier);
	QFile f_options(le_chemin);
	if (f_options.exists()){
		f_options.open( QIODevice::ReadOnly );
		QTextStream ts( &f_options );
		int valeur_1, valeur_2, valeur_3, valeur_4, valeur_5, valeur_6, valeur_7;
		ts >> valeur_1; ts >> valeur_2; ts >> valeur_3; ts >> valeur_4; ts >> valeur_5; ts >> valeur_6; ts >> valeur_7;
		if( valeur_1>=0 && valeur_1<=1 && valeur_2>=0 && valeur_2<=1 && valeur_3>=0 && valeur_3<=1
		 && valeur_4>=min_temps_reflexion && valeur_4<=max_temps_reflexion && valeur_5>=min_nb_plaques && valeur_5<=max_nb_plaques
		 && valeur_6>=min_min_compte && valeur_6<=max_min_compte && valeur_7>=min_max_compte && valeur_7<=max_max_compte
		 && valeur_6<=valeur_7){
			calcul::cano1_actif=valeur_1;
			calcul::cano2_actif=valeur_2;
			calcul::cano3_actif=valeur_3;
			WidgetHorloge::nb_sec=valeur_4;
			tirage::nb_plaques=valeur_5;
			tirage::compte_minimal=valeur_6;
			tirage::compte_maximal=valeur_7;
		}
		else cout << "Fichier d'options foireux" << endl;
		f_options.close();
	}
	//fin du chargement du fichier d'options
	mise_a_zero();
	demarrage=false;
	calc_resolu=false;
}

//fonction d'ouverture pour le cochage des options
void Principal::options() {
	if(calc_en_cours) return;	// on bloque le menu options tant que le calcul n'est pas fini.
	
	int ancien_nb_plaques=tirage::nb_plaques;
	int ancien_compte_minimal=tirage::compte_minimal;
	int ancien_compte_maximal=tirage::compte_maximal;
	calc_resolu=false;
	alea=false;
	//barre de progression
	horloge->arrete();
	horloge->remise_a_zero();
	
	WidgetOptions opt( this );
	opt.exec();
	
	if(ancien_nb_plaques!=tirage::nb_plaques || ancien_compte_minimal!=tirage::compte_minimal || ancien_compte_maximal!=tirage::compte_maximal){
		mise_a_zero();
	}
}

//fonction d'ouverture de la fentre d'aide
void Principal::helpAide() {
	lire_texte(2);
}

//fonction d'ouverture de la fentre de license
void Principal::helpLicence() {
	lire_texte(1);
}

//fonction d'ouverture de la fentre " propos"
void Principal::helpAbout() {
	lire_texte(0);
}

void Principal::lire_texte(int num) {
	if(num==0) {
		WidgetPopup popup( this, false, QString(":/txt/about.txt"), QString("A propos de Kitsune"));
		popup.exec();
	} 
	else if(num==1) {
		//popup de licence (true)
		WidgetPopup popup( this, true, QString(":/txt/licence.txt"), QString("Licence de Kitsune"));
		popup.exec();
	} 
	else if(num==2) {
		WidgetPopup popup( this, false, QString(":/txt/aide.txt"), QString("Aide"));
		popup.exec();
	}
}

//une fonction pour mettre  jour l'affichage des plaques
void Principal::update_affichage_plaques() {
	for(int Wi=0; Wi<6; Wi++) {
		emit refresh(Wi, liste_des_plaques[Wi]);
	}
}

//une fonction pour mettre  jour l'affichage du compte
void Principal::update_affichage_compte() {
	for(int Wi=0; Wi<3; Wi++) {
		emit refresh(Wi+10, liste_des_chiffres[Wi]);
	}
}

//une fonction pour mettre  jour l'affichage du statut, du renard et du texte
void Principal::update_affichage_statut() {
	BarreStatut->showMessage( texte_statut_attente );
	pixmapLabelRenard->setPixmap(*renard_normal);
	texte_renard->setText( texte_renard_repos );
}

//lance le thread de rsolution du problme
void Principal::resoudre() {
	//attention  ne pas relancer un calcul avant que le prcdent soit fini
	if (calc_en_cours) return;
	//si le calcul a dj t rsolu, inutile de le relancer - on affiche les solutions pour le cas o on viendrait d'appuyer sur "Alatoire"
	if (calc_resolu){
		horloge->arrete();
		affiche_solutions();
		return;
	}
	//on met ce boolen  true jusqu' ce que l'tude du tirage lance ci-dessous soit termine
	calc_en_cours=true;
	
	//affichage
	BarreStatut->showMessage( QString(texte_statut_en_cours) );
	setCursor( QCursor(Qt::WaitCursor));		//la souris est un sablier pendant le calcul
	
	//initialisation du compteur pour connaitre le temps de calcul
	heure->start();
	
	//calcul du compte  tudier
	compte le_compte_etudie=liste_des_chiffres[0]*100+liste_des_chiffres[1]*10+liste_des_chiffres[2];
	
	//tude du tirage
	J->init(t,le_compte_etudie,le_compte_etudie);
	J->start();	//lance etude_tirage sous forme d'un thread
}

//appel automatiquement en fin de calcul (connect au signal J.finished)
void Principal::fin_calcul() {
	//dtermination du temps que le calcul a pris
	int temps_ecoule=heure->elapsed();
	
	//paramtres d'tat
	calc_en_cours=false;
	calc_resolu=true;
	if(alea==false) affiche_solutions();
	alea=false;				//remise a false de alea dans tous les cas
	
	//calcul termin => on rtablit la flche de la souris
	setCursor( QCursor(Qt::ArrowCursor));
	
	//calcul et affichage du temps coul
	int secondes=temps_ecoule/1000;
	int centiemes=(temps_ecoule%1000)/10;
	QString resultat=QString(texte_statut_duree);
	resultat.append( QString().sprintf("%d,", secondes) );
	if(centiemes<10){		//pour viter la blague du 1.7s au lieu de 1.07s
		resultat.append( QString("0"));
	}
	resultat.append( QString().sprintf("%d s", centiemes) );
	
	BarreStatut->showMessage( resultat );
}

//affichage des solutions
void Principal::affiche_solutions() {
	//test au cas o la barre de progression arrive  0 avant que l'tude du tirage soit termine (machine TRES lente)
	if(calc_en_cours){
		alea=false;
		return;
	}
	
	//rappel du tirage et du compte
	textEditB->setPlainText("");
	textEditB->insertPlainText(QString("Tirage : "));
	textEditB->insertPlainText(QString(J->afficheProbleme().c_str()));
	textEditB->insertPlainText(QString("\n\n"));

	//affichage des rsultats dans le cadre de texte prvu  cet effet
	textEditB->insertPlainText(QString(J->afficheBase().c_str()));
	//petite phrase d'information en bas de la fentre + image
	if(J->distance_a_la_solution()==0){
		texte_renard->setText( texte_renard_ceb );
		pixmapLabelRenard->setPixmap(*renard_heureux);
	}
	else{
		texte_renard->setText( texte_renard_pds );
		pixmapLabelRenard->setPixmap(*renard_triste);
	}
}

//dtermination alatoire d'un nouveau problme
void Principal::aleatoire(){
	//tat de l'application
	if (calc_en_cours) return;		//on dsactive la touche si un calcul est en cours
	calc_resolu=false;
	alea=true;
	textEditB->setPlainText("");		//rinitialisation de la zone des rsultats
	horloge->lance();
	
	//dtermination du nouveau compte
	liste_des_chiffres.clear();
	liste_des_chiffres.assign(3,0);
	
	int valeur_compte=tirage::compte_minimal+rand()%(tirage::compte_maximal-tirage::compte_minimal+1);
	
	liste_des_chiffres[0]=valeur_compte/100;
	liste_des_chiffres[1]=(valeur_compte/10)%10;
	liste_des_chiffres[2]=(valeur_compte)%10;
	
	//mini-animation avant d'afficher le compte
	anim();
	
	//affichage du nouveau compte
	update_affichage_compte();
	
	//dtermination des plaques et du tirage
	t.tab.clear();
	
	pseudo_tirage pt;  //pseudo-tirage pour le calcul alatoire des plaques
	pt.assign(tirage::nb_plaques,0);
	liste_des_plaques.clear();
	liste_des_plaques.assign(tirage::nb_plaques,0);
	int i;
	int hasard;
	vector<int> v;
	v.assign(24,0);
	for(i=0;i<=23;i++){
		v[i]=i;
	}

	for(i=0;i<tirage::nb_plaques;i++){
		hasard=rand()%(24-i);
		pt[i]=v[hasard];
		v.erase(v.begin()+hasard);
		switch (pt[i]){
			case 23: liste_des_plaques[i]=100; break;
			case 22: liste_des_plaques[i]=75; break;
			case 21: liste_des_plaques[i]=50; break;
			case 20: liste_des_plaques[i]=25; break;
			default: liste_des_plaques[i]=(1+pt[i]/2); break;
		}
	}
	t=pseudo_en_tirage(pt);
	
	//affichage des plaques et statut
	update_affichage_plaques();
	update_affichage_statut();
	
	//lancement de la resolution en tache de fond
	resoudre();
}

void Principal::anim() {
	//a completer
}


//remise de l'application dans l'tat initial (hormis le texte d'accueil)
void Principal::mise_a_zero(){
	//tat de l'application
	if (calc_en_cours) return;		//on dsactive la touche si un calcul est en cours
	calc_resolu=false;
	
	//initialisation des textes et images
	if(!demarrage){
		update_affichage_statut();
	}
	textEditB->setPlainText("");
	BarreStatut->showMessage( texte_statut_attente );
	
	//barre de progression pour le temps coul
	horloge->arrete();
	horloge->remise_a_zero();
	
	//initialisation du compte  la valeur du compte minimal
	liste_des_chiffres.clear();
	liste_des_chiffres.assign(3,0);
	liste_des_chiffres[0]=tirage::compte_minimal/100;
	liste_des_chiffres[1]=(tirage::compte_minimal/10)%10;
	liste_des_chiffres[2]=tirage::compte_minimal%10;
	
	//affichage du compte
	update_affichage_compte();
	
	//initialisation des plaques
	int i;
	liste_des_plaques.clear();
	liste_des_plaques.assign(tirage::nb_plaques,0);
	for(i=0;i<tirage::nb_plaques;i++){
		liste_des_plaques[i]=i+1;
	}
	
	//affichage des plaques
	update_affichage_plaques();
	
	//initialisation du tirage
	t.tab.clear();
	for(i=1;i<=tirage::nb_plaques;i++){
		t.tab[i]++;
	}
}

//fonction appele  chaque modification du compte
void Principal::compte_change(int numero_plaque, bool est_plus) {
	//tat de l'application
	if (calc_en_cours) return;		//on dsactive les touches si un calcul est en cours
	calc_resolu=false;
	
	//barre de progression
	horloge->arrete();
	horloge->remise_a_zero();
	
	if(est_plus) {
		liste_des_chiffres[numero_plaque-10]=liste_des_chiffres[numero_plaque-10]+1;
	} else {
		liste_des_chiffres[numero_plaque-10]=liste_des_chiffres[numero_plaque-10]-1;
	}
	
	int nouveau_compte=liste_des_chiffres[0]*100+liste_des_chiffres[1]*10+liste_des_chiffres[2];
	
	if(nouveau_compte>=tirage::compte_minimal && nouveau_compte <= tirage::compte_maximal) {
		liste_des_chiffres[0]=nouveau_compte/100;
		liste_des_chiffres[1]=(nouveau_compte/10)%10;
		liste_des_chiffres[2]=(nouveau_compte)%10;
		update_affichage_compte();
		update_affichage_statut();
		return;
	}
	
	//annulation de la modif
	if(est_plus) {
		liste_des_chiffres[numero_plaque-10]=liste_des_chiffres[numero_plaque-10]-1;
	} else {
		liste_des_chiffres[numero_plaque-10]=liste_des_chiffres[numero_plaque-10]+1;
	}
}

//fonction appele  chaque modification d'une plaque
void Principal::plaque_change(int numero_plaque, bool est_plus) {
	//tat de l'application
	if (calc_en_cours) return;		//on dsactive les touches si un calcul est en cours
	calc_resolu=false;
	
	//barre de progression
	horloge->arrete();
	horloge->remise_a_zero();
	
	plaque plaque_passee=liste_des_plaques[numero_plaque];
	plaque plaque_future=0;
	if(est_plus){
		plaque_future=plus_petite_plaque_sup(t,plaque_passee);
	} else{
		plaque_future=plus_petite_plaque_inf(t,plaque_passee);
	}
	
	if(plaque_future!=0){
		liste_des_plaques[numero_plaque]=plaque_future;
		t.tab[plaque_passee]--;
		t.tab[plaque_future]++;
	
		//affichage
		update_affichage_plaques();
		update_affichage_statut();  
	}
}

